################################################################################
#
#   Stock reports module for OpenERP
#   Copyright (C) 2009, Almacom (Thailand) Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

from osv import fields,osv

class stock_location(osv.osv):
    _name="stock.location"
    _inherit="stock.location"

    def _stock_value(self,cr,uid,ids,name,args,context={}):
        vals={}
        for loc_id in ids:
            cr.execute("select greatest(stock_val(%s),0.0)",(loc_id,))
            r=cr.fetchone()
            vals[loc_id]=r[0]
        return vals

    _columns = {
        "stock_value": fields.function(_stock_value,method=True,type="float",string="Stock Value"),
    }
stock_location()

class hr_department(osv.osv):
    _inherit="hr.department"

    _columns={
        "stock_value": fields.related("stock_location_id","stock_value",type="float",string="Stock Value"),
    }
hr_department()

class stock_report_tracking(osv.osv):
    _name = "stock.report.tracking"
    _description = "Stock report by tracking lots"
    _auto = False
    _columns = {
            'name': fields.float('Current Stock', readonly=True),
            'qty_virt': fields.float('Future Stock', readonly=True),
            'location_id': fields.many2one('stock.location', 'Location', readonly=True, select=True),
            'product_id': fields.many2one('product.product', 'Product', readonly=True, select=True),
            'tracking_id': fields.many2one('stock.tracking', 'Tracking lot', readonly=True, select=True),
    }
    _order="location_id,product_id"
    def init(self, cr):
        cr.execute("""
            create or replace view stock_report_tracking as (
                select pl.id as id,
                    pl.location_id,
                    pl.product_id,
                    pl.tracking_id,
                    pl.qty as name,
                    pl.qty_virt
                from product_location_tracking pl
                left join product_product prod on prod.id=pl.product_id
                left join stock_tracking track on track.id=pl.tracking_id
            )""")
stock_report_tracking()

class stock_report(osv.osv):
    _name = "stock.report"
    _description = "Stock report"
    _auto = False
    _columns = {
            'name': fields.float('Current Stock', readonly=True),
            'qty_virt': fields.float('Future Stock', readonly=True),
            'location_id': fields.many2one('stock.location', 'Location', readonly=True, select=True),
            'product_id': fields.many2one('product.product', 'Product', readonly=True, select=True),
    }
    _order="location_id,product_id"
    def init(self, cr):
        cr.execute("""
            create or replace view stock_report as (
                select pl.id,
                    pl.location_id,
                    pl.product_id,
                    pl.qty as name,
                    pl.qty_virt
                from product_location pl
                left join product_product prod on prod.id=product_id
            )""")
stock_report()

class report_stock_val_time(osv.osv):
    _name="report.stock.val.time"
    _auto=False
    _columns={
        "date":fields.char("Date",size=64,readonly=True,select=True),
        "location_id":fields.many2one("stock.location","Location",readonly=True),
        "val_first":fields.float("First Stock Value Of Day",readonly=True),
        "val_last":fields.float("Last Stock Value Of Day",readonly=True),
        "val_min":fields.float("Min Stock Value Of Day",readonly=True),
        "val_max":fields.float("Max Stock Value Of Day",readonly=True),
    }
    _order="date desc,location_id"

    def init(self,cr):
        cr.execute("""
create or replace view report_stock_val_time as (
    select id_ as id, to_char(date_,'YYYY-MM-DD') as date, location_id__ as location_id, first_ as val_first, last_ as val_last, min_ as val_min, max_ as val_max from stock_val_hist_day(90)
)""")
report_stock_val_time()

