create language plpgsql;

create or replace view product_location as
    select max(id) as id,location_id,product_id,sum(qty) as qty,sum(qty_virt) as qty_virt from (
        select -max(sm.id) as id,
            sm.location_id,
            sm.product_id,
            -sum(sm.product_qty) as qty,
            0.0 as qty_virt
        from stock_move as sm
        where state = 'done'
        group by sm.location_id, sm.product_id, sm.product_uom
        union all
        select max(sm.id) as id,
            sm.location_dest_id as location_id,
            sm.product_id,
            sum(sm.product_qty) as qty,
            0.0 as qty_virt
        from stock_move as sm
        where sm.state = 'done'
        group by sm.location_dest_id, sm.product_id, sm.product_uom
        union all
        select -max(sm.id) as id,
            sm.location_id as location_id,
            sm.product_id,
            0.0 as qty,
            -sum(sm.product_qty) as qty_virt
        from stock_move as sm
        where sm.state in ('confirmed','waiting','assigned','done')
        group by sm.location_id, sm.product_id, sm.product_uom
        union all
        select max(sm.id) as id,
            sm.location_dest_id as location_id,
            sm.product_id,
            0.0 as qty,
            sum(sm.product_qty) as qty_virt
        from stock_move as sm
        where sm.state in ('confirmed','waiting','assigned','done')
        group by sm.location_dest_id, sm.product_id, sm.product_uom
    ) as report group by location_id,product_id having sum(qty)!=0.0 or sum(qty_virt)!=0.0;

create or replace view product_location_tracking as
    select max(id) as id,location_id,product_id,tracking_id,sum(qty) as qty,sum(qty_virt) as qty_virt from (
        select -max(sm.id) as id,
            sm.location_id,
            sm.product_id,
            sm.tracking_id,
            -sum(sm.product_qty) as qty,
            0.0 as qty_virt
        from stock_move as sm
        where state = 'done'
        group by sm.location_id, sm.product_id, sm.tracking_id, sm.product_uom
        union all
        select max(sm.id) as id,
            sm.location_dest_id as location_id,
            sm.product_id,
            sm.tracking_id,
            sum(sm.product_qty) as qty,
            0.0 as qty_virt
        from stock_move as sm
        where sm.state = 'done'
        group by sm.location_dest_id, sm.product_id, sm.tracking_id, sm.product_uom
        union all
        select -max(sm.id) as id,
            sm.location_id as location_id,
            sm.product_id,
            sm.tracking_id,
            0.0 as qty,
            -sum(sm.product_qty) as qty_virt
        from stock_move as sm
        where sm.state in ('confirmed','waiting','assigned','done')
        group by sm.location_id, sm.product_id, sm.tracking_id, sm.product_uom
        union all
        select max(sm.id) as id,
            sm.location_dest_id as location_id,
            sm.product_id,
            sm.tracking_id,
            0.0 as qty,
            sum(sm.product_qty) as qty_virt
        from stock_move as sm
        where sm.state in ('confirmed','waiting','assigned','done')
        group by sm.location_dest_id, sm.product_id, sm.tracking_id, sm.product_uom
    ) as report group by location_id,product_id,tracking_id having sum(qty)!=0.0 or sum(qty_virt)!=0.0;

create or replace function stock_val_history(out id_ integer, out location_id_ integer, out time_ timestamp, out val_ float) returns setof record as $$
declare
    total_val float;
    rec record;
begin
    for location_id_ in select id from stock_location loop
        total_val:=0.0;
        for rec in select max(id) as id,time,sum(val) as val from (
            select -move.id as id ,move.date_planned as time,-move.product_qty*prodt.standard_price as val
                from stock_move move
                left join product_product prod on prod.id=move.product_id
                left join product_template prodt on prodt.id=prod.product_tmpl_id
                where move.location_id=location_id_ and move.state='done'
            union all
            select move.id as id,move.date_planned as time,move.product_qty*prodt.standard_price as val
                from stock_move move
                left join product_product prod on prod.id=move.product_id
                left join product_template prodt on prodt.id=prod.product_tmpl_id
                where location_dest_id=location_id_ and move.state='done'
        ) as vals group by time having sum(val)!=0.0 order by time
        loop
            total_val:=total_val+rec.val;
            id_:=rec.id;
            time_:=rec.time;
            val_:=total_val;
            if val_<0 then -- XXX datrus
                val_:=0;
            end if;
            return next;
        end loop;
    end loop;
end;
$$ language plpgsql;

create or replace function stock_val_hist_day(num_days integer, out id_ integer, out date_ date, out location_id__ integer, out first_ float, out last_ float, out min_ float, out max_ float) returns setof record as $$
declare
    i integer;
    t0 timestamp;
    t1 timestamp;
    rec record;
begin
    create temporary table hist as select * from stock_val_history();
    id_:=1;
    date_:='today'::date;
    for i in 1..num_days loop
        for location_id__ in select id from stock_location loop
            t0:=date_::timestamp;
            t1:=t0+'1 day'::interval;
            select val_ into first_ from hist where location_id_=location_id__ and time_<=t0 order by time_ desc limit 1;
            select val_ into last_ from hist where location_id_=location_id__ and time_<t1 order by time_ desc limit 1;
            select min(val_),max(val_) into rec from hist where location_id_=location_id__ and time_>=t0 and time_<t1;
            min_:=rec.min;
            max_:=rec.max;
            if min_ is null or first_<min_ then
                min_:=first_;
            end if;
            if max_ is null or first_>max_ then
                max_:=first_;
            end if;
            return next;
            id_:=id_+1;
        end loop;
        date_:=date_-'1 day'::interval;
    end loop;
    drop table hist;
end;
$$ language plpgsql;

create or replace function stock_val(loc_id integer) returns float as $$
    select sum(val) from (
        select -move.product_qty*prodt.standard_price as val
            from stock_move move
            left join product_product prod on prod.id=move.product_id
            left join product_template prodt on prodt.id=prod.product_tmpl_id
            where move.location_id=$1 and move.state='done'
        union all
        select move.product_qty*prodt.standard_price as val
            from stock_move move
            left join product_product prod on prod.id=move.product_id
            left join product_template prodt on prodt.id=prod.product_tmpl_id
            where move.location_dest_id=$1 and move.state='done'
    ) as vals
$$ language sql;
